! -*- coding: utf-8 -*-
!<
!! @file
!! @brief Ajustement polynomial
!!
!! @ingroup  ALG_AjustementPolynomial
!! @defgroup ALG_AjustementPolynomial ALG_AjustementPolynomial: Ajustement polynomial
!!
!! <H1>Description</H1>
!!
!! Ajustement de fonctions sinusoidales et polynomiales.
!!

C $Id: ALG_AjustementPolynomial.F 5514 2011-11-07 15:30:22Z ppoilbarbe $



C******************************************************************************
C******************************************************************************
!>
!! @brief Ajustement de fonctions sinusoidales et polynomiales
!!
!! Ce sous-programme effectue l'ajustement d'un polynôme
!! et de fonctions sinusoïdales.
!!
!! Soit A la matrice des estimations des fonctions aux points considérés.
!!
!! Soit C le vecteur des coefficients.
!!
!! Soit Y le vecteur des ordonnées (observations).
!!
!! On doit minimiser  ||AC - y|| * ||AC - Y||, ce qui revient à
!! l'équation A<sup>T</sup>.A.C = A<sup>T</sup>.Y
!!
!! @param [in]     Ijv_pol         présence (1) ou absence (0) d'un polynôme
!! @param [in]     Ijv_nb_deg      nombre de degrés du polynôme
!! @param [in]     Ijv_nb_pul      nombre de pulsations à ajuster
!! @param [in]     Idw_pul         valeur des pulsations
!! @param [in]     Ijv_nb_mes      nombre de valeurs dans tableaux suivants
!! @param [in]     Ijv_nb_coef     nombre de valeurs dans tableaux suivants
!! @param [in]     Idw_x           tableau des abscisses
!! @param [in]     Idw_y           tableau des ordonnées
!! @param [in]     Idw_mat_a       matrice A
!! @param [in]     Idw_mat_at      matrice At
!! @param [in]     Idw_mat_ata     matrice At.a
!! @param [in]     Idw_mat_buff    matrice ((At.A)-1 . At)
!! @param [out]    Odw_coeff       tableau des coefficients de l'ajustement
!! @param [out]    Odw_y_ajust     tableau des ordonnées apres ajustement
!! @param [out]    Ojv_Status      Compte rendu de traitement
      subroutine ALG_Function_Adjust(Ijv_pol,
     &                               Ijv_nb_deg,
     &                               Ijv_nb_pul,
     &                               Idw_pul,
     &                               Ijv_nb_mes,
     &                               Ijv_nb_coef,
     &                               Idw_x,
     &                               Idw_y,
     &                               Idw_mat_a,
     &                               Idw_mat_at,
     &                               Idw_mat_ata,
     &                               Idw_mat_buff,
     &                               Odw_coeff,
     &                               Odw_y_ajust,
     &                               jv_status)

      implicit none
      integer*4      Ijv_pol
      integer*4      Ijv_nb_deg
      integer*4      Ijv_nb_pul
      real*8         Idw_pul(Ijv_nb_pul)
      integer*4      Ijv_nb_mes
      integer*4      Ijv_nb_coef
      real*8         Idw_x(Ijv_nb_mes)
      real*8         Idw_y(Ijv_nb_mes)
      real*8         Idw_mat_a(Ijv_nb_mes, Ijv_nb_coef)
      real*8         Idw_mat_at(Ijv_nb_coef, Ijv_nb_mes)
      real*8         Idw_mat_ata(Ijv_nb_mes, Ijv_nb_mes)
      real*8         Idw_mat_buff(Ijv_nb_coef, Ijv_nb_mes)
C     tableau des coefficients dans l'ordre suivant :
C       constante, coef pol deg 1, coef pol deg 2,...
C       coef cos 1, coef sin 1, coef cos 2, ...
      real*8         Odw_coeff(Ijv_nb_coef)
      real*8         Odw_y_ajust(Ijv_nb_mes)
      integer*4      jv_status



 1010 format(A)
 9999 continue
        write(0, 1010) ' '
        write(0, 1010) '************* ERREUR **********'
        write(0, 1010) ' '
        write(0, 1010) '----- Pile d''appel'
        write(0, 1010), tc_nomop
        return
      end


!> @cond

C Fonctions locales
C +--------------------------------------------------------------------+
C !                                                                    !
C ! TRANS_MAT                                                          !
C !                                                                    !
C +--------------------------------------------------------------------+
C !                                                                    !
C ! ROLE: Calcul matriciel : calcule At (transposee de A) a partir de A!
C ! ----                                                               !
C !                                                                    !
C ! PARAMETRES EN ENTREE                                               !
C !                                                                    !
C !    ENTREE:                                                         !
C !      dw_mat_A    : matrice de depart A                             !
C !      jv_dim1     : dimension memoire de la matrice (lignes)        !
C !      jv_dim2     : dimension memoire de la matrice (colonnes)      !
C !      jv_nlig     : nombre de lignes                                !
C !      jv_ncol     : nombre de colonnes                              !
C !                                                                    !
C ! DONNEES EN SORTIE                                                  !
C !                                                                    !
C !      dw_mat_At   : matrice At                                      !
C !                                                                    !
C +--------------------------------------------------------------------+
C !                                                                    !
C !                                      P. Mambert                    !
C !                                      Fevrier 1997                  !
C +--------------------------------------------------------------------+

      subroutine trans_mat (dw_mat_A,
     &                      jv_dim1,
     &                      jv_dim2,
     &                      dw_mat_At)

      implicit none

      integer*4      jv_dim1                    ! (E)
      integer*4      jv_dim2                    ! (E)
      real*8         dw_mat_A(jv_dim1, jv_dim2) ! (E)
      real*8         dw_mat_At(jv_dim2, jv_dim1)! (S)


      return
      end


C +--------------------------------------------------------------------+
C !                                                                    !
C ! MULT_MAT                                                           !
C !                                                                    !
C +--------------------------------------------------------------------+
C !                                                                    !
C ! ROLE: Produit matriciel : calcule A.B a partir de A et B           !
C ! ----                                                               !
C !                                                                    !
C ! PARAMETRES EN ENTREE                                               !
C !                                                                    !
C !    ENTREE:                                                         !
C !      dw_mat_A    : matrice de depart A                             !
C !      jv_dimA1    : dimension memoire de A (lignes)                 !
C !      jv_dimA2    : dimension memoire de A (colonnes)               !
C !      jv_nligA    : nombre de lignes                                !
C !      jv_ncolA    : nombre de colonnes                              !
C !      dw_mat_B    : matrice de depart B                             !
C !      jv_dimB1    : dimension memoire de B (lignes)                 !
C !      jv_dimB2    : dimension memoire de B (colonnes)               !
C !      jv_nligB    : nombre de lignes                                !
C !      jv_ncolB    : nombre de colonnes                              !
C !      jv_status   : compte-rendu d'execution                        !
C !                                                                    !
C ! DONNEES EN SORTIE                                                  !
C !                                                                    !
C !      dw_mat_AB   : matrice AB = A.B                                !
C !                                                                    !
C +--------------------------------------------------------------------+
C !                                                                    !
C !                                      P. Mambert                    !
C !                                      Fevrier 1997                  !
C +--------------------------------------------------------------------+
      subroutine mult_mat  (dw_mat_A,
     &                      jv_dimA1,
     &                      jv_dimA2,
     &                      dw_mat_B,
     &                      jv_dimB1,
     &                      jv_dimB2,
     &                      dw_mat_AB,
     &                      jv_status)

      implicit none


      integer*4      jv_dimA1                           ! (E)
      integer*4      jv_dimA2                           ! (E)
      real*8         dw_mat_A(jv_dimA1, jv_dimA2)       ! (E)
      integer*4      jv_dimB1                           ! (E)
      integer*4      jv_dimB2                           ! (E)
      real*8         dw_mat_B(jv_dimB1, jv_dimB2)       ! (E)
      real*8         dw_mat_AB(jv_dimA1, jv_dimB2)      ! (S)
      integer*4      jv_status                          ! (S)


 9999 continue
      return
      end


C +--------------------------------------------------------------------+
C !                                                                    !
C ! INV_MAT                                                            !
C !                                                                    !
C +--------------------------------------------------------------------+
C !                                                                    !
C ! ROLE: Inversion de la matrice A : calcule A-1 a partir de A        !
C ! ----                                                               !
C !                                                                    !
C ! PARAMETRES EN ENTREE                                               !
C !                                                                    !
C !    ENTREE:                                                         !
C !      dw_mat_A    : matrice de depart A                             !
C !      jv_ordre    : ordre de la matrice                             !
C !                                                                    !
C ! DONNEES EN SORTIE                                                  !
C !                                                                    !
C !      dw_mat_A    : matrice inverse A-1                             !
C !      jv_status   : compte-rendu d'execution                        !
C !                                                                    !
C ! Utilisation de la librairie FORTRAN NAG                            !
C ! (routines DPOTRF et DPOTRI)                                        !
C +--------------------------------------------------------------------+
C !                                                                    !
C !                                      P. Mambert                    !
C !                                      Fevrier 1997                  !
C +--------------------------------------------------------------------+

      subroutine inv_mat  (jv_ordre,
     &                     dw_mat_A,
     &                     jv_status)

      implicit none

C  +--------------------------------------------------------------------------+
C  +           D E C L A R A T I O N   D E S   P A R A M E T R E S            +
C  +--------------------------------------------------------------------------+

      integer*4      jv_ordre                           ! (E)
      real*8         dw_mat_A(jv_ordre, jv_ordre)       ! (E/S)
      integer*4      jv_status                          ! (S)

C  +--------------------------------------------------------------------------+
C  +                      V A R I A B L E S   L O C A L E S                   +
C  +--------------------------------------------------------------------------+

C     parametres d'appel a la fct SPOTRI
      integer*4      jv_info

      integer*2      ev_i
      integer*2      ev_j

C  +--------------------------------------------------------------------------+
C  +                                C O D E                                   +
C  +--------------------------------------------------------------------------+

      jv_status = jc_su_glob_ok


C     Factoriser A
C     ------------
      call dpotrf ( 'U',
     &              jv_ordre,
     &              dw_mat_A,
     &              jv_ordre,
     &              jv_info)

      if (jv_info .ne. 0) then
        print *, 'Erreur INV_MAT:DPOTRF: ',jv_info
        jv_status = jc_su_glob_nok
        goto 9999
      endif



 9999 continue
      return
      end

!> @endcond
